define([
    "dijit/_Container",
    "dijit/_WidgetBase",
    "dijit/registry",
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dnd/Source",
    "dojo/dnd/common",
    "dojo/dom-construct",
    "dojo/on"
], function(_Container, _WidgetBase, registry, array, declare, lang, Source, dnd, domConstruct, on) {
    return declare([_WidgetBase,_Container],{
        name: "",
        value: null,
        multiple: true,
        type: "text",
        /* container : dojo/dnd/Container or subclass
         * determine which container to use for the list
         */
        container: Source,
        buildRendering: function() {
            this.inherited(arguments);

            var sourceParams = {};
            var paramsToInherit = [
                "singular",
                "creator",
                "skipForm",
                "dropParent",
                "isSource",
                "autoSync",
                "copyOnly",
                "delay",
                "horizontal",
                "selfCopy",
                "selfAccept",
                "withHandles",
                "generateText"
            ];
            array.forEach(paramsToInherit, function(param){
                if ( typeof this[param] !== "undefined" ) {
                    sourceParams[param] = this[param];
                }
            },this);
            lang.mixin(sourceParams, {
                accept: [this.type],
                creator: lang.hitch(this, "creator"),
                dropParent: this.containerNode
            });
            this.source = new this.container(this.domNode,sourceParams);
            this.own(this.source.on('Drop',lang.hitch(this,'_handleChange')));
        },
        creator: function(item, hint) {
            var id = dnd.getUniqueId();
            var nodeOrWidget = null;
            if ( hint === "avatar" ) {
                if ( this.createAvatar ) {
                    nodeOrWidget = this.createAvatar(id,item);
                } else {
                    return this.source.defaultCreator(item, hint);
                }
            } else {
                if ( this.createListElement ) {
                    nodeOrWidget = this.createListElement(id,item);
                } else {
                    return this.source.defaultCreator(item, hint);
                }
            }
            this.own(on(nodeOrWidget,'change',lang.hitch(this,'_handleChange')));
            var node = nodeOrWidget.domNode ? nodeOrWidget.domNode : nodeOrWidget;
            if ( hint !== "avatar" && node.id !== id ) {
                console.warn("Node id '%s' not equal to generated id '%s'. Is this intended?", node.id, id);
            }
            return {
                data: item,
                type: [this.type],
                node: node
            };
        },
        createAvatar: null, /*function(id,item){},*/
        createListElement: null, /* function(id,item){},*/
        _getValueAttr: function() {
            this.value = array.map(this.source.getAllNodes(),function(node){
                var widget = registry.byNode(node);
                if ( widget && "value" in widget ) {
                    return widget.get('value');
                } else {
                    return this.source.getItem(node.id).data;
                }
            },this);
            return this.value;
        },
        _setValueAttr: function(value) {
            this.clear();
            this.appendItems(value || []);
        },
        _setDisabledAttr: function(value) {
            this._set("disabled", value);
            array.forEach(this.getChildren(), function(child){
                child.set("disabled", value);
            });
        },
        _setReadOnlyAttr: function(value) {
            this._set("readOnly", value);
            array.forEach(this.getChildren(), function(child){
                child.set("readOnly", value);
            });
        },
        focus: function() {
            var children = this.getChildren();
            if ( children.length > 0 ) {
                children[0].focus();
            }
        },
        appendItems: function(items,forceEvent) {
            this.source.insertNodes(false,items);
            if ( forceEvent ) { this._handleChange(); }
        },
        appendItem: function(item,forceEvent) {
            this.source.insertNodes(false,[item]);
            if ( forceEvent ) { this._handleChange(); }
        },
        removeItem: function(key,forceEvent) {
            array.forEach(array.filter(this.source.getAllNodes(), function(node){
                return node.id === key;
            }), lang.hitch(this, "_destroyNodeOrWidget"));
            this.source.delItem(key);
            if ( forceEvent ) { this._handleChange(); }
        },
        getChildren: function() {
            return array.filter(array.map(this.source.getAllNodes(), function(node){
                return registry.byNode(node);
            }),function(widget){ return widget !== null; });
        },
        clear: function(forceEvent) {
            array.forEach(this.source.getAllNodes(),
                          lang.hitch(this, "_destroyNodeOrWidget"));
            this.source.clearItems();
            if ( forceEvent ) { this._handleChange(); }
        },
        _destroyNodeOrWidget: function(node) {
            var widget = registry.byNode(node);
            if ( widget ) {
                widget.destroyRecursive();
            } else {
                domConstruct.destroy(node);
            }
        },
        validate: function() {
            return array.every(array.map(this.source.getAllNodes(),function(node){
                var widget = registry.byNode(node);
                return !widget || widget.disabled || !widget.validate || widget.validate();
            }), function(item){ return item; });
        },
        destroy: function() {
            this.source.destroy();
            this.inherited(arguments);
        },
        _handleChange: function() {
            this.value = this._getValueAttr();
            this.onChange(this.value);
        },
        onChange: function(value) {}
    });
});
